from xlte.amari import kpi as akpi
from xlte import kpi
import io
import numpy as np

def load_measurements(alogm):
  mlog = kpi.MeasurementLog()
  try:
    while 1:
      m = alogm.read()
      if m is None:
        break
      mlog.append(m)
  finally:
    alogm.close()
  return mlog

def calc_each_period(mlog, tperiod):
  t = mlog.data()[0]['X.Tstart']
  for m in mlog.data()[1:]:
    t_ = m['X.Tstart']
    if (t_ - t) >= tperiod:
      calc = kpi.Calc(mlog, t, t + tperiod)
      t = calc.tau_hi
      yield calc

def getORSKPIValue(data):
  fxlog = io.StringIO(data)
  alogm = akpi.LogMeasure(fxlog,  open('/dev/null', 'r'))
  mlog = load_measurements(alogm)
  vt = []
  vInititialEPSBEstabSR = []
  vAddedEPSBEstabSR     = []

  for calc in calc_each_period(mlog, 60): # 60 seconds
    vt.append(calc.tau_lo)
    x = calc.erab_accessibility()        # E-RAB Accessibility
    vInititialEPSBEstabSR.append((x[0]['lo'], x[0]['hi']))
    vAddedEPSBEstabSR.append((x[1]['lo'], x[1]['hi']))

  vt = np.asarray([x for x in vt])
  vInititialEPSBEstabSR = np.asarray(vInititialEPSBEstabSR)
  vAddedEPSBEstabSR = np.asarray(vAddedEPSBEstabSR)
 
  # E-UTRAN IP Throughput KPI.
  evt = []
  vIPThp_qci = []

  for calc in calc_each_period(mlog, 3): # 3 seconds
    evt.append(calc.tau_lo)
    x = calc.eutran_ip_throughput()     # E-UTRAN IP Throughput
    tmp = []
    for i in x:
      tmp.append((i['dl']['lo'], i['dl']['hi'], i['ul']['lo'], i['ul']['hi']))
    vIPThp_qci.append(tmp)

  evt = np.asarray([x for x in evt])
  vIPThp_qci = vIPThp_qci
  return vt, vInititialEPSBEstabSR, vAddedEPSBEstabSR, evt, vIPThp_qci
