/*global window, rJS, RSVP, console, Plotly */
/*jslint nomen: true, indent: 2 */
(function (window, rJS, RSVP, Plotly, wendelin) {
  "use strict";

  var gadget_klass = rJS(window);

  function getColumn(array, column) {
    return array.map(function (value, index) {
      return value[column];
    });
  }
  function allEqual(array, value) {
    return array.every(function (current) {
      return current === value;
    });
  }

  function getPlotData(lo, hi, date) {
    var color,
      color_array = [
        {'color': '#1f77b4'},
        {'color': '#ff7f0e'},
        {'color': '#2ca02c'},
        {'color': '#d62728'},
        {'color': '#9467bd'},
        {'color': '#8c564b'},
        {'color': '#e377c2'},
        {'color': '#7f7f7f'},
        {'color': '#bcbd22'},
        {'color': '#17becf'}
      ],
      length = lo[0].length,
      column_value,
      data_list = [],
      i;

    for (i = 0; i < length; i += 1) {
      column_value = getColumn(hi, i);
      if (allEqual(column_value, 0)) {
        continue;
      }
      color = color_array[i % color_array.length];
      data_list.push({
        x: date,
        marker: {
          size: 4
        },
        mode: 'lines+markers',
        y: getColumn(lo, i),
        type: 'scatter',
        line: color,
        name: 'IPThp.' + i,
        hovertemplate: 'Date: %{x}<br>Download: %{y} Mbit/s'
      });
      data_list.push({
        x: date,
        mode: 'lines',
        y: column_value,
        type: 'scatter',
        line: color,
        opacity: 0.3,
        showlegend: false,
        fill: 'tonexty',
        hoverinfo: 'none'
      });
    }
    return data_list;
  }
  gadget_klass
    .declareAcquiredMethod("getSetting", "getSetting")
    .declareAcquiredMethod("jio_get", "jio_get")
    .declareAcquiredMethod("jio_getAttachment", "jio_getAttachment")
    .ready(function (gadget) {
      gadget.property_dict = {};
    })
    .declareMethod('render', function (option_dict) {
      var gadget = this,
        i,
        download_data,
        upload_data,
        label_list = ["evt", "dl_lo", "dl_hi", "ul_lo", "ul_hi"];
      return new RSVP.Queue().push(function () {
          return wendelin.getArrayRawSlice(gadget, option_dict.data_array_key);
        })
        .push(function (result) {
          var graph_data = nj.unpack(result.pick( null, label_list)),
            date = [],
            dl_lo = [],
            dl_hi = [],
            ul_lo = [],
            ul_hi = [],
            raw_dl_lo,
            raw_dl_hi,
            raw_ul_lo,
            raw_ul_hi;
          for (i = 0; i < graph_data.length; i += 1) {
            if (date.indexOf(graph_data[i][0]) == -1) {
              date.push(graph_data[i][0]);
              dl_lo.push(raw_dl_lo);
              dl_hi.push(raw_dl_hi);
              ul_lo.push(raw_ul_lo);
              ul_hi.push(raw_ul_hi);
              raw_dl_lo = [graph_data[i][1] / 1e6];
              raw_dl_hi = [graph_data[i][2] / 1e6];
              raw_ul_lo = [graph_data[i][3] / 1e6];
              raw_ul_hi = [graph_data[i][4] / 1e6];
            } else {
              raw_dl_lo.push(graph_data[i][1] / 1e6);
              raw_dl_hi.push(graph_data[i][2] / 1e6);
              raw_ul_lo.push(graph_data[i][3] / 1e6);
              raw_ul_hi.push(graph_data[i][4] / 1e6);
            }
          }
          dl_lo.push(raw_dl_lo);
          dl_hi.push(raw_dl_hi);
          ul_lo.push(raw_ul_lo);
          ul_hi.push(raw_ul_hi);
          dl_lo.shift();
          dl_hi.shift();
          ul_lo.shift();
          ul_hi.shift();
          date.forEach(function (element, index) {
            date[index] = new Date(element * 1000);
          });
          download_data = getPlotData(dl_lo, dl_hi, date);
          upload_data = getPlotData(ul_lo, ul_hi, date);
          Plotly.newPlot(
            gadget.element.querySelector('.download-link'),
            download_data,
            {
              /*
              'legend': {
                'x': 0,
                'y': 1
              },*/
              'title' : 'Download Link',
              'xaxis': {
                'autorange': true
                /*
                'rangeslider': {
                    //range: [date[0], date[date.length - 1]]
                }*/
              },
              'yaxis': {
                title: {
                  text: 'Mbit/s'
                }
              }
            }
          );

          Plotly.newPlot(
             gadget.element.querySelector('.upload-link'),
             upload_data,
             {
               /*
              'legend': {
                'x': 0,
                'y': 1
              },*/
              'title' : 'Upload Link',
              'xaxis': {
                'autorange': true
                /*
                'rangeslider': {
                  'range': [date[0], date[date.length - 1]]
                }*/
              },
              'yaxis': {
                title: {
                  text: 'Mbit/s'
                }
              }
            });
          //XXXXXXXXXXXX
          gadget.element.querySelector('.ui-icon-spinner').hidden = true;
        });
    });

}(window, rJS, RSVP, Plotly, wendelin));
