##############################################################################
#
# Copyright (c) 2002-2015 Nexedi SA and Contributors. All Rights Reserved.
#
# This program is free software: you can Use, Study, Modify and Redistribute
# it under the terms of the GNU General Public License version 3, or (at your
# option) any later version, as published by the Free Software Foundation.
#
# You can also Link and Combine this program with other software covered by
# the terms of any of the Free Software licenses or any of the Open Source
# Initiative approved licenses and Convey the resulting work. Corresponding
# source of such a combination shall include the source code for all other
# software used.
#
# This program is distributed WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See COPYING file for full licensing terms.
# See https://www.nexedi.com/licensing for rationale and options.
#
##############################################################################
from Products.ERP5Type.tests.ERP5TypeTestCase import ERP5TypeTestCase


class ORSWendelinTest(ERP5TypeTestCase):
  """
  ORS Wendelin Test
  """

  def getTitle(self):
    return "ORS Wendelin Test"

  def afterSetUp(self):
    test_data_acquisition_unit = getattr(self.portal.data_acquisition_unit_module, 'test_data_acquisition_unit', None)
    if not test_data_acquisition_unit:
      test_data_acquisition_unit = self.portal.data_acquisition_unit_module.newContent(
        portal_type='Data Acquisition Unit',
        id='test_data_acquisition_unit',
        reference='test_data_acquisition_unit')

    if test_data_acquisition_unit.getValidationState() != 'validated':
      test_data_acquisition_unit.validate()

    data_stream = self._getDataStream(test_data_acquisition_unit)
    if data_stream:
      data_stream.setData(None)

    self.test_data_acquisition_unit = test_data_acquisition_unit
    self.test_ors_example_log = self.portal.web_page_module.test_ors_example_log
    self.tic()

  def _getDataStream(self, obj):
    for line in obj.getAggregateRelatedValueList(portal_type='Data Ingestion Line'):
      data_stream = line.getAggregateValue(portal_type='Data Stream')
      if data_stream:
        return data_stream

  def _ingestData(self, reference, data):
    request = self.portal.REQUEST
    request.environ["REQUEST_METHOD"] = 'POST'
    request.set('reference', reference)
    request.set('data_chunk', data)
    self.portal.portal_ingestion_policies.ors_ingestion.ingest()
    self.tic()

  def test_ingestion(self):
    data_supply = self.test_data_acquisition_unit.DataAcquisitionUnit_createORSDataSupply(batch=1)
    self.tic()
    self.assertEqual(data_supply.getValidationState(), 'validated')
    self._ingestData(data_supply.getReference(), self.test_ors_example_log.getTextContent())
    self.tic()
    data_stream = self._getDataStream(self.test_data_acquisition_unit)
    self.assertTrue(data_stream.getData() is not None)
    # first one
    self.portal.portal_alarms.wendelin_handle_analysis.activeSense()
    self.tic()
    self.portal.portal_alarms.wendelin_handle_analysis.activeSense()
    self.tic()
    # resample
    self.portal.portal_alarms.wendelin_handle_analysis.activeSense()
    self.tic()
    self.portal.portal_alarms.wendelin_handle_analysis.activeSense()
    self.tic()
    self.assertTrue(self.test_data_acquisition_unit.Base_getERABDataArrayKey() is not None)
    self.assertTrue(self.test_data_acquisition_unit.Base_getEUTRANDataArrayKey() is not None)




  def test_transformation(self):
    pass